function [p,A]=pLUinplace_Vec(A)
% permuted LU factorization for N-by-n-by-n 3D-array A,  
%   see Golub G.H & Van Loan C.F., Matrix Computations (3rd E.d) Algorithm 3.4.1, page 112 
%     P_k*A_k=L_k*U_k   where A_k=A(k,:,:) 
% Returns 
%   - A, the modify n-by-n matrix such that 
%       L=fc_linalg3D.tril(A,-1)+fc_linalg3D.eye(N,n), U=fc_linalg3D.triu(A)
%     One ca also use [L,U]=fc_linalg3D.fact.LUfrominplace(A)
%   - p, the permutation indices N-by-n array such that the n-by-n permutation matrix P_k is defined by
%        P_k(i,p(k,i))=1 for all i in 1:n
%        P_k(i,j)=0 otherwize
%    in Matlab/Octave to build the N-by-n-by-n 3D-array P: 
%       P=zeros(N,n,n); 
%       P(sub2ind(size(P),repmat([1:N]',1,n),repmat(1:n,N,1),p))=1
%    or
%       P=fc_linalg3D.fact.permInd2Mat(p)
  assert( fc_linalg3D.is3Darray(A) )
  [N,m,n]=size(A);
  assert( m==n )
  p=repmat(1:m,N,1);
  K=[1:N]';
  Subs=@(I,J) sub2ind([N,m,n],K,I,J);
  II=repmat([1:m],N,1);
  for i=1:n-1
    % -> pivot/permutation
    [~,idx]=max(abs(A(:,i:n,i)),[],2);
    mu=idx+i-1;
    for j=1:n
      idx1=Subs(mu,II(:,j));idx2=Subs(II(:,i),II(:,j));
      A([idx1;idx2])=A([idx2;idx1]);
    end
    idx1=sub2ind([N,m],K,mu);idx2=sub2ind([N,m],K,II(:,i));
    p([idx1;idx2])=p([idx2;idx1]);
    % <- pivot/permutation
    I=abs(A(:,i,i))>1e-15;
    rows=i+1:n;
%      A(I,rows,i)=A(I,rows,i)./A(I,i,i);
%      A(I,rows,i+1:n)=A(I,rows,i+1:n)-A(I,rows,i).*A(I,i,i+1:n);
    A=Elimination(A,I,rows,i);
  end
  if nargout==1, p={p,A};end
end


function A=Elimination(A,I,rows,i)
  n=size(A,3);
  if ~fc_tools.comp.isOctave() && verLessThan('matlab','9.1.0')  % 2016b
    cols=i+1:n;
    A(I,rows,i)=A(I,rows,i)./repmat(A(I,i,i),[1,length(rows)]);
    for j=cols
      A(I,rows,j)=A(I,rows,j)-A(I,rows,i).*repmat(A(I,i,j),[1,length(rows)]);
    end
  else
    A(I,rows,i)=A(I,rows,i)./A(I,i,i);
    A(I,rows,i+1:end)=A(I,rows,i+1:end)-A(I,rows,i).*A(I,i,i+1:end);
  end
end
